@@ -50,7 +50,7 @@ CommandLine::CommandLine(void)
 , parfilename()
 , rawfilenames()
 , extrafiles()
-, operation(opNone)  
+, operation(opNone)
 , purgefiles(false)
 , skipdata(false)
 , skipleaway(0)
@@ -134,8 +134,9 @@ void CommandLine::usage(void)
     "  -f<n>    : First Recovery-Block-Number\n"
     "  -u       : Uniform recovery file sizes\n"
     "  -l       : Limit size of recovery files (don't use both -u and -l)\n"
-    "  -n<n>    : Number of recovery files (don't use both -n and -l)\n"
+    "  -n<n>    : Number of recovery files (max 31) (don't use both -n and -l)\n"
     "  -R       : Recurse into subdirectories\n"
+    "             (Be aware of wildcard shell expansion)\n"
     "\n";
   cout <<
     "Example:\n"
@@ -157,12 +158,12 @@ bool CommandLine::Parse(int argc, const char * const *argv)
     if (!ComputeBlockSize())
       return false;
 
-    u64 sourceblockcount = 0;
+    u32 sourceblockcount = 0;
     u64 largestfilesize = 0;
     for (vector<string>::const_iterator i=extrafiles.begin(); i!=extrafiles.end(); i++)
     {
       u64 filesize = filesize_cache.get(*i);
-      sourceblockcount += (filesize + blocksize-1) / blocksize;
+      sourceblockcount += (u32) ((filesize + blocksize-1) / blocksize);
       if (filesize > largestfilesize)
       {
 	largestfilesize = filesize;
@@ -183,7 +184,7 @@ bool CommandLine::Parse(int argc, const char * const *argv)
       return false;
     }
   }
-  
+
   return true;
 }
 
@@ -469,12 +470,12 @@ bool CommandLine::ReadArgs(int argc, const char * const *argv)
             else
             {
               const char *p = &argv[0][2];
-              while (redundancy <= 10 && *p && isdigit(*p))
+              while (*p && isdigit(*p))
               {
                 redundancy = redundancy * 10 + (*p - '0');
                 p++;
               }
-              if (redundancy > 100 || *p)
+              if (*p)
               {
                 cerr << "Invalid redundancy option: " << argv[0] << endl;
                 return false;
@@ -484,6 +485,10 @@ bool CommandLine::ReadArgs(int argc, const char * const *argv)
                 cerr << "Cannot set redundancy to 0 and file count > 0" << endl;
                 return false;
               }
+	      if (redundancy > 100)
+	      {
+	        cerr << "WARNING: Creating recovery file(s) with " << redundancy << "% redundancy." << endl;
+	      }
             }
             redundancyset = true;
           }
@@ -566,7 +571,7 @@ bool CommandLine::ReadArgs(int argc, const char * const *argv)
               cerr << "Invalid option: " << argv[0] << endl;
               return false;
             }
-            if (recoveryfilescheme != scUnknown)
+            if (recoveryfilescheme != scUnknown && recoveryfilescheme != scUniform)
             {
               cerr << "Cannot specify two recovery file size schemes." << endl;
               return false;
@@ -638,6 +643,23 @@ bool CommandLine::ReadArgs(int argc, const char * const *argv)
               cerr << "Invalid recovery file count option: " << argv[0] << endl;
               return false;
             }
+
+            // cap the maximum number of recovery files
+            // because the number of recovery blocks will be
+            // (2 ^ recoveryfilecount) - 1
+            // the number 32 will overflow the u32 resulting in 1
+            if (recoveryfilecount > 31)
+            {
+              cerr << "Invalid recovery file count option: " << recoveryfilecount << endl;
+              cerr << "  the maximum allowed recovery file count is 31" << endl;
+
+              return false;
+            }
+
+            // When number of recovery files is used, set the recoveryfilescheme
+            // to uniform, since variable will not always be able to fill all
+            // the files
+            recoveryfilescheme = scUniform;
           }
           break;
 
@@ -805,7 +827,7 @@ bool CommandLine::ReadArgs(int argc, const char * const *argv)
 	      cerr << "  (Run \"" << path << name << " --help\" for supported options.)" << endl;
               return false;
             }
-	      
+
             argc--;
             argv++;
             options = false;
@@ -867,12 +889,12 @@ bool CommandLine::ReadArgs(int argc, const char * const *argv)
 u64 CommandLine::GetTotalPhysicalMemory()
 {
   u64 TotalPhysicalMemory = 0;
-  
+
   HMODULE hLib = ::LoadLibraryA("kernel32.dll");
   if (NULL != hLib)
   {
     BOOL (WINAPI *pfn)(LPMEMORYSTATUSEX) = (BOOL (WINAPI*)(LPMEMORYSTATUSEX))::GetProcAddress(hLib, "GlobalMemoryStatusEx");
-    
+
     if (NULL != pfn)
     {
       MEMORYSTATUSEX mse;
@@ -882,16 +904,16 @@ u64 CommandLine::GetTotalPhysicalMemory()
 	TotalPhysicalMemory = mse.ullTotalPhys;
       }
     }
-    
+
     ::FreeLibrary(hLib);
   }
-  
+
   if (TotalPhysicalMemory == 0)
   {
     MEMORYSTATUS ms;
     ::ZeroMemory(&ms, sizeof(ms));
     ::GlobalMemoryStatus(&ms);
-    
+
     TotalPhysicalMemory = ms.dwTotalPhys;
   }
 
@@ -902,7 +924,7 @@ u64 CommandLine::GetTotalPhysicalMemory()
 u64 CommandLine::GetTotalPhysicalMemory()
 {
   long pages = sysconf(_SC_PHYS_PAGES);
-  long page_size = sysconf(_SC_PAGESIZE);  
+  long page_size = sysconf(_SC_PAGESIZE);
   return pages*page_size;
 }
 #else
@@ -921,7 +943,7 @@ bool CommandLine::CheckValuesAndSetDefaults() {
     return false;
   }
 
-  
+
   // Default noise level
   if (noiselevel == nlUnknown)
   {
@@ -943,6 +965,11 @@ bool CommandLine::CheckValuesAndSetDefaults() {
     // Half of total physical memory
     memorylimit = (size_t)(TotalPhysicalMemory / 1048576 / 2);
   }
+
+  // limit to 1GB on 32-bit platforms to avoid exhausing the addressable memory space
+  if (sizeof(uintptr_t) < 8 && memorylimit > 1024)
+    memorylimit = 1024;
+
   // convert to megabytes
   memorylimit *= 1048576;
 
@@ -1030,13 +1057,13 @@ bool CommandLine::CheckValuesAndSetDefaults() {
     } //end file exists
   }
 
-  
-  // operation should alway be set, but let's be thorough.
+
+  // operation should always be set, but let's be thorough.
   if (operation == opNone) {
     cerr << "ERROR: No operation was specified (create, repair, or verify)" << endl;
     return false;
   }
-  
+
 
   if (operation != opCreate) {
     // skipdata is bool and either value is valid.
@@ -1092,6 +1119,12 @@ bool CommandLine::CheckValuesAndSetDefaults() {
       parfilename = parfilename.substr(0, parfilename.length()-5);
     }
 
+    if (DiskFile::FileExists(parfilename + ".par2"))
+    {
+      cerr << "Par2 file already exists: " << parfilename << endl;
+      return false;
+    }
+
     // If neither block count not block size is specified
     if (blockcount == 0 && blocksize == 0)
     {
@@ -1213,7 +1246,7 @@ bool CommandLine::ComputeBlockSize() {
       }
     }
   }
-  
+
   return true;
 }
 
@@ -1250,7 +1283,7 @@ bool CommandLine::ComputeRecoveryBlockCount(u32 *recoveryblockcount,
     {
       u32 estimatedFileCount = 15;
       u64 overhead = estimatedFileCount * overhead_per_recovery_file;
-      u64 estimatedrecoveryblockcount;
+      u32 estimatedrecoveryblockcount;
       if (overhead > redundancysize)
       {
         estimatedrecoveryblockcount = 1;  // at least 1
@@ -1306,7 +1339,6 @@ bool CommandLine::ComputeRecoveryBlockCount(u32 *recoveryblockcount,
     return false;
   }
 
-  cout << endl;
   return true;
 }
 
@@ -1317,6 +1349,8 @@ bool CommandLine::ComputeRecoveryBlockCount(u32 *recoveryblockcount,
 bool CommandLine::SetParFilename(string filename)
 {
   bool result = false;
+
+#ifndef _WIN32
   string::size_type where;
 
   if ((where = filename.find_first_of('*')) != string::npos ||
@@ -1325,6 +1359,7 @@ bool CommandLine::SetParFilename(string filename)
     cerr << "par2 file must not have a wildcard in it." << endl;
     return result;
   }
+#endif
 
   // If we are verifying or repairing, the PAR2 file must
   // already exist
