@@ -29,7 +29,7 @@ static char THIS_FILE[]=__FILE__;
 #endif
 
 
-// static variable 
+// static variable
 #ifdef _OPENMP
 u32 Par2Creator::filethreads = _FILE_THREADS;
 #endif
@@ -65,7 +65,7 @@ Par2Creator::Par2Creator(std::ostream &sout, std::ostream &serr, const NoiseLeve
 , rs()
 , progress(0)
 , totaldata(0)
-  
+
 , deferhashcomputation(false)
 #ifdef _OPENMP
 , mttotalsize(0)
@@ -107,7 +107,10 @@ Result Par2Creator::Process(
 #ifdef _OPENMP
   filethreads = _filethreads;
 #endif
-  
+
+  if (!CheckBasepath(parfilename))
+    return eFileIOError;
+
   // Get information from commandline
   blocksize = _blocksize;
   const vector<string> extrafiles = _extrafiles;
@@ -143,6 +146,9 @@ Result Par2Creator::Process(
   if (!CalculateProcessBlockSize(memorylimit))
     return eLogicError;
 
+  if (recoveryblockcount > 0 && noiselevel >= nlDebug)
+    sout << "[DEBUG] Process chunk size: " << chunksize << endl;
+
   if (noiselevel > nlQuiet)
   {
     // Display information.
@@ -171,7 +177,7 @@ Result Par2Creator::Process(
   if (!CreateSourceBlocks())
     return eLogicError;
 
-  // Create all of the output files and allocate all packets to appropriate file offets.
+  // Create all of the output files and allocate all packets to appropriate file offsets.
   if (!InitialiseOutputFiles(parfilename))
     return eFileIOError;
 
@@ -236,12 +242,30 @@ Result Par2Creator::Process(
   return eSuccess;
 }
 
+// Check basepath permission
+bool Par2Creator::CheckBasepath(const string &parfilename)
+{
+  string checkfilename = parfilename + ".check.par2";
+  DiskFile *diskfile = new DiskFile(sout, serr);
+  size_t dummysize = 4096;
+
+  if (!diskfile->Create(checkfilename, dummysize))
+    return false;
+
+  diskfile->Close();
+
+  if (!diskfile->Delete())
+    return false;
+
+  return true;
+}
+
 // Compute block size from block count or vice versa depending on which was
 // specified on the command line
 bool Par2Creator::ComputeBlockCount(const vector<string> &extrafiles)
 {
   FileSizeCache filesize_cache;
-  
+
   largestfilesize = 0;
   for (vector<string>::const_iterator i=extrafiles.begin(); i!=extrafiles.end(); i++)
   {
@@ -251,7 +275,7 @@ bool Par2Creator::ComputeBlockCount(const vector<string> &extrafiles)
       largestfilesize = filesize;
     }
   }
-  
+
 
   if (blocksize == 0)
   {
@@ -293,7 +317,7 @@ bool Par2Creator::CalculateProcessBlockSize(size_t memorylimit)
   if (recoveryblockcount == 0)
   {
     chunksize = 0;
-    
+
     deferhashcomputation = false;
   }
   else
@@ -312,6 +336,12 @@ bool Par2Creator::CalculateProcessBlockSize(size_t memorylimit)
 
       deferhashcomputation = true;
     }
+    
+    if (MAX_CHUNK_SIZE != 0 && chunksize > MAX_CHUNK_SIZE)
+    {
+      chunksize = MAX_CHUNK_SIZE;
+      deferhashcomputation = false;
+    }
   }
 
   return true;
@@ -423,7 +453,7 @@ bool Par2Creator::CreateSourceBlocks(void)
        sourcefile!= sourcefiles.end();
        sourcefile++)
   {
-    // Allocate the appopriate number of source blocks to each source file.
+    // Allocate the appropriate number of source blocks to each source file.
     // sourceblock will be advanced.
 
     (*sourcefile)->InitialiseSourceBlocks(sourceblock, blocksize);
@@ -447,7 +477,7 @@ public:
   u32 count;
 };
 
-// Create all of the output files and allocate all packets to appropriate file offets.
+// Create all of the output files and allocate all packets to appropriate file offsets.
 bool Par2Creator::InitialiseOutputFiles(const string &parfilename)
 {
   // Allocate the recovery packets
@@ -743,7 +773,7 @@ bool Par2Creator::ProcessData(u64 blockoffset, size_t blocklength)
   // Clear the output buffer
   memset(outputbuffer, 0, chunksize * recoveryblockcount);
 
-  // If we have defered computation of the file hash and block crc and hashes
+  // If we have deferred computation of the file hash and block crc and hashes
   // sourcefile and sourceindex will be used to update them during
   // the main recovery block computation
   vector<Par2CreatorSourceFile*>::iterator sourcefile = sourcefiles.begin();
@@ -868,7 +898,7 @@ bool Par2Creator::WriteRecoveryPacketHeaders(void)
 
 bool Par2Creator::FinishFileHashComputation(void)
 {
-  // If we defered the computation of the full file hash, then we finish it now
+  // If we deferred the computation of the full file hash, then we finish it now
   if (deferhashcomputation)
   {
     // For each source file
