@@ -2,6 +2,7 @@
 //  repair tool). See http://parchive.sourceforge.net for details of PAR 2.0.
 //
 //  Copyright (c) 2003 Peter Brian Clements
+//  Copyright (c) 2019 Michael D. Nahas
 //
 //  par2cmdline is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -48,15 +49,17 @@ public:
   ~ReedSolomon(void);
 
   // Set which input blocks are present or missing
-  bool SetInput(const vector<bool> &present); // Some input blocks are present
-  bool SetInput(u32 count);                   // All input blocks are present
+  // Some input blocks are present
+  bool SetInput(const vector<bool> &present, std::ostream &sout, std::ostream &serr);
+  // All input blocks are present
+  bool SetInput(u32 count, std::ostream &sout, std::ostream &serr);
 
   // Set which output block are available or need to be computed
   bool SetOutput(bool present, u16 exponent);
   bool SetOutput(bool present, u16 lowexponent, u16 highexponent);
 
   // Compute the RS Matrix
-  bool Compute(CommandLine::NoiseLevel noiselevel);
+  bool Compute(NoiseLevel noiselevel, std::ostream &sout, std::ostream &serr);
 
   // Process a block of data
   bool Process(size_t size,             // The size of the block of data
@@ -69,8 +72,10 @@ private:
 
 protected:
   // Perform Gaussian Elimination
-  bool GaussElim(CommandLine::NoiseLevel noiselevel,
-                 unsigned int rows,
+  bool GaussElim(NoiseLevel noiselevel,
+		 std::ostream &sout,
+		 std::ostream &serr,
+		 unsigned int rows,
                  unsigned int leftcols,
                  G *leftmatrix,
                  G *rightmatrix,
@@ -104,10 +109,16 @@ protected:
 #ifdef LONGMULTIPLY
   GaloisLongMultiplyTable<g> *glmt;  // A multiplication table used by Process()
 #endif
+
+private:
+  // private copy constructor to prevent any misuse.
+  ReedSolomon(const ReedSolomon &);
+  ReedSolomon& operator=(const ReedSolomon &);
 };
 
 template<class g>
 inline ReedSolomon<g>::ReedSolomon(void)
+: outputrows()
 {
   inputcount = 0;
 
@@ -117,8 +128,6 @@ inline ReedSolomon<g>::ReedSolomon(void)
   datamissingindex = 0;
   database = 0;
 
-  outputrows.empty();
-
   outputcount = 0;
 
   parpresent = 0;
@@ -203,31 +212,33 @@ inline bool ReedSolomon<g>::SetOutput(bool present, u16 lowexponent, u16 highexp
 
 // Construct the Vandermonde matrix and solve it if necessary
 template<class g>
-inline bool ReedSolomon<g>::Compute(CommandLine::NoiseLevel noiselevel)
+inline bool ReedSolomon<g>::Compute(NoiseLevel noiselevel, std::ostream &sout, std::ostream &serr)
 {
   u32 outcount = datamissing + parmissing;
   u32 incount = datapresent + datamissing;
 
   if (datamissing > parpresent)
   {
-    cerr << "Not enough recovery blocks." << endl;
+    serr << "Not enough recovery blocks." << endl;
     return false;
   }
   else if (outcount == 0)
   {
-    cerr << "No output blocks." << endl;
+    serr << "No output blocks." << endl;
     return false;
   }
 
-  if (noiselevel > CommandLine::nlQuiet)
-    cout << "Computing Reed Solomon matrix." << endl;
+  if (noiselevel > nlQuiet)
+    sout << "Computing Reed Solomon matrix." << endl;
 
   /*  Layout of RS Matrix:
+      NOTE: The second set of columns represents the parity vectors present,
+      but this only uses datamissing of them.  Otherwise, it is over constrained.
 
-                                       parpresent
-                     datapresent       datamissing         datamissing       parmissing
+                                       datamissing
+                     datapresent      <=parpresent        datamissing       parmissing
                /                     |             \ /                     |           \
-   parpresent  |           (ppi[row])|             | |           (ppi[row])|           |
+               |           (ppi[row])|             | |           (ppi[row])|           |
    datamissing |          ^          |      I      | |          ^          |     0     |
                |(dpi[col])           |             | |(dmi[col])           |           |
                +---------------------+-------------+ +---------------------+-----------+
@@ -260,10 +271,10 @@ inline bool ReedSolomon<g>::Compute(CommandLine::NoiseLevel noiselevel)
   {
     // Define MPDL to skip reporting and speed things up
 #ifndef MPDL
-    if (noiselevel > CommandLine::nlQuiet)
+    if (noiselevel > nlQuiet)
     {
       int progress = row * 1000 / (datamissing+parmissing);
-      cout << "Constructing: " << progress/10 << '.' << progress%10 << "%\r" << flush;
+      sout << "Constructing: " << progress/10 << '.' << progress%10 << "%\r" << flush;
     }
 #endif
 
@@ -307,10 +318,10 @@ inline bool ReedSolomon<g>::Compute(CommandLine::NoiseLevel noiselevel)
   {
     // Define MPDL to skip reporting and speed things up
 #ifndef MPDL
-    if (noiselevel > CommandLine::nlQuiet)
+    if (noiselevel > nlQuiet)
     {
       int progress = (row+datamissing) * 1000 / (datamissing+parmissing);
-      cout << "Constructing: " << progress/10 << '.' << progress%10 << "%\r" << flush;
+      sout << "Constructing: " << progress/10 << '.' << progress%10 << "%\r" << flush;
     }
 #endif
 
@@ -348,15 +359,15 @@ inline bool ReedSolomon<g>::Compute(CommandLine::NoiseLevel noiselevel)
 
     outputrow++;
   }
-  if (noiselevel > CommandLine::nlQuiet)
-    cout << "Constructing: done." << endl;
+  if (noiselevel > nlQuiet)
+    sout << "Constructing: done." << endl;
 
   // Solve the matrices only if recovering data
   if (datamissing > 0)
   {
     // Perform Gaussian Elimination and then delete the right matrix (which
     // will no longer be required).
-    bool success = GaussElim(noiselevel, outcount, incount, leftmatrix, rightmatrix, datamissing);
+    bool success = GaussElim(noiselevel, sout, serr, outcount, incount, leftmatrix, rightmatrix, datamissing);
     delete [] rightmatrix;
     return success;
   }
@@ -366,37 +377,37 @@ inline bool ReedSolomon<g>::Compute(CommandLine::NoiseLevel noiselevel)
 
 // Use Gaussian Elimination to solve the matrices
 template<class g>
-inline bool ReedSolomon<g>::GaussElim(CommandLine::NoiseLevel noiselevel, unsigned int rows, unsigned int leftcols, G *leftmatrix, G *rightmatrix, unsigned int datamissing)
+inline bool ReedSolomon<g>::GaussElim(NoiseLevel noiselevel, std::ostream &sout, std::ostream &serr, unsigned int rows, unsigned int leftcols, G *leftmatrix, G *rightmatrix, unsigned int datamissing)
 {
-  if (noiselevel >= CommandLine::nlDebug)
+  if (noiselevel >= nlDebug)
   {
     for (unsigned int row=0; row<rows; row++)
     {
-      cout << ((row==0) ? "/"    : (row==rows-1) ? "\\"    : "|");
+      sout << ((row==0) ? "/"    : (row==rows-1) ? "\\"    : "|");
       for (unsigned int col=0; col<leftcols; col++)
       {
-        cout << " "
+        sout << " "
              << hex << setw(G::Bits>8?4:2) << setfill('0')
              << (unsigned int)leftmatrix[row*leftcols+col];
       }
-      cout << ((row==0) ? " \\ /" : (row==rows-1) ? " / \\" : " | |");
+      sout << ((row==0) ? " \\ /" : (row==rows-1) ? " / \\" : " | |");
       for (unsigned int col=0; col<rows; col++)
       {
-        cout << " "
+        sout << " "
              << hex << setw(G::Bits>8?4:2) << setfill('0')
              << (unsigned int)rightmatrix[row*rows+col];
       }
-      cout << ((row==0) ? " \\"   : (row==rows-1) ? " /"    : " | |");
-      cout << endl;
+      sout << ((row==0) ? " \\"   : (row==rows-1) ? " /"    : " | |");
+      sout << endl;
 
-      cout << dec << setw(0) << setfill(' ');
+      sout << dec << setw(0) << setfill(' ');
     }
   }
 
   // Because the matrices being operated on are Vandermonde matrices
   // they are guaranteed not to be singular.
 
-  // Additionally, because Galois arithmetic is being used, all calulations
+  // Additionally, because Galois arithmetic is being used, all calculations
   // involve exact values with no loss of precision. It is therefore
   // not necessary to carry out any row or column swapping.
 
@@ -415,7 +426,7 @@ inline bool ReedSolomon<g>::GaussElim(CommandLine::NoiseLevel noiselevel, unsign
     assert(pivotvalue != 0);
     if (pivotvalue == 0)
     {
-      cerr << "RS computation error." << endl;
+      serr << "RS computation error." << endl;
       return false;
     }
 
@@ -444,13 +455,13 @@ inline bool ReedSolomon<g>::GaussElim(CommandLine::NoiseLevel noiselevel, unsign
     {
       // Define MPDL to skip reporting and speed things up
 #ifndef MPDL
-      if (noiselevel > CommandLine::nlQuiet)
+      if (noiselevel > nlQuiet)
       {
         int newprogress = (row*rows+row2) * 1000 / (datamissing*rows);
         if (progress != newprogress)
         {
           progress = newprogress;
-          cout << "Solving: " << progress/10 << '.' << progress%10 << "%\r" << flush;
+          sout << "Solving: " << progress/10 << '.' << progress%10 << "%\r" << flush;
         }
       }
 #endif
@@ -501,30 +512,30 @@ inline bool ReedSolomon<g>::GaussElim(CommandLine::NoiseLevel noiselevel, unsign
       }
     }
   }
-  if (noiselevel > CommandLine::nlQuiet)
-    cout << "Solving: done." << endl;
-  if (noiselevel >= CommandLine::nlDebug)
+  if (noiselevel > nlQuiet)
+    sout << "Solving: done." << endl;
+  if (noiselevel >= nlDebug)
   {
     for (unsigned int row=0; row<rows; row++)
     {
-      cout << ((row==0) ? "/"    : (row==rows-1) ? "\\"    : "|");
+      sout << ((row==0) ? "/"    : (row==rows-1) ? "\\"    : "|");
       for (unsigned int col=0; col<leftcols; col++)
       {
-        cout << " "
+        sout << " "
              << hex << setw(G::Bits>8?4:2) << setfill('0')
              << (unsigned int)leftmatrix[row*leftcols+col];
       }
-      cout << ((row==0) ? " \\ /" : (row==rows-1) ? " / \\" : " | |");
+      sout << ((row==0) ? " \\ /" : (row==rows-1) ? " / \\" : " | |");
       for (unsigned int col=0; col<rows; col++)
       {
-        cout << " "
+        sout << " "
              << hex << setw(G::Bits>8?4:2) << setfill('0')
              << (unsigned int)rightmatrix[row*rows+col];
       }
-      cout << ((row==0) ? " \\"   : (row==rows-1) ? " /"    : " | |");
-      cout << endl;
+      sout << ((row==0) ? " \\"   : (row==rows-1) ? " /"    : " | |");
+      sout << endl;
 
-      cout << dec << setw(0) << setfill(' ');
+      sout << dec << setw(0) << setfill(' ');
     }
   }
 
